"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Options = exports.Docker = exports.dockerCommand = void 0;
const child_process = __importStar(require("child_process"));
const cli_table_2_json_1 = require("cli-table-2-json");
const dockermachine_cli_js_1 = require("dockermachine-cli-js");
//import * as _ from 'lodash';
const snakeCase = require("lodash.snakecase");
const nodeify_ts_1 = __importDefault(require("nodeify-ts"));
const exec = child_process.exec;
const splitLines = function (input) {
    return input.replace(/\r/g, '').split('\n');
};
const array2Oject = function (lines) {
    return lines.reduce(function (object, linep) {
        const line = linep.trim();
        if (line.length === 0) {
            return object;
        }
        const parts = line.split(':');
        let key = parts[0];
        const value = parts.slice(1).join(':');
        key = snakeCase(key);
        object[key] = value.trim();
        return object;
    }, {});
};
const extractResult = function (result) {
    const extracterArray = [
        {
            re: / build /,
            run(resultp) {
                const lines = splitLines(resultp.raw);
                lines.forEach(function (line) {
                    const re = /Successfully built (.*)$/;
                    const str = line;
                    const m = re.exec(str);
                    if (m !== null) {
                        if (m.index === re.lastIndex) {
                            re.lastIndex++;
                        }
                        // View your result using the m-variable.
                        // eg m[0] etc.
                        resultp.success = true;
                        resultp.imageId = m[1];
                    }
                });
                resultp.response = lines;
                return resultp;
            },
        },
        {
            re: / run /,
            run(resultp) {
                resultp.containerId = resultp.raw.trim();
                return resultp;
            },
        },
        {
            re: / ps /,
            run(resultp) {
                const lines = splitLines(resultp.raw);
                resultp.containerList = (0, cli_table_2_json_1.cliTable2Json)(lines);
                return resultp;
            },
        },
        {
            re: / images /,
            run(resultp) {
                const lines = splitLines(resultp.raw);
                //const debug = require('debug')('docker-cli-js:lib/index.js extractResult images');
                //debug(lines);
                resultp.images = (0, cli_table_2_json_1.cliTable2Json)(lines);
                return resultp;
            },
        },
        {
            re: / network ls /,
            run(resultp) {
                const lines = splitLines(resultp.raw);
                //const debug = require('debug')('docker-cli-js:lib/index.js extractResult images');
                //debug(lines);
                resultp.network = (0, cli_table_2_json_1.cliTable2Json)(lines);
                return resultp;
            },
        },
        {
            re: / inspect /,
            run(resultp) {
                const object = JSON.parse(resultp.raw);
                resultp.object = object;
                return resultp;
            },
        },
        {
            re: / info /,
            run(resultp) {
                const lines = splitLines(resultp.raw);
                resultp.object = array2Oject(lines);
                return resultp;
            },
        },
        {
            re: / search /,
            run(resultp) {
                const lines = splitLines(resultp.raw);
                resultp.images = (0, cli_table_2_json_1.cliTable2Json)(lines);
                return resultp;
            },
        },
        {
            re: / login /,
            run(resultp) {
                resultp.login = resultp.raw.trim();
                return resultp;
            },
        },
        {
            re: / pull /,
            run(resultp) {
                resultp.login = resultp.raw.trim();
                return resultp;
            },
        },
        {
            re: / push /,
            run(resultp) {
                resultp.login = resultp.raw.trim();
                return resultp;
            },
        },
    ];
    extracterArray.forEach(function (extracter) {
        const re = extracter.re;
        const str = result.command;
        const m = re.exec(str + ' ');
        if (m !== null) {
            if (m.index === re.lastIndex) {
                re.lastIndex++;
            }
            // View your result using the m-constiable.
            // eg m[0] etc.
            return extracter.run(result);
        }
    });
    return result;
};
const dockerCommand = (command, options = {
    currentWorkingDirectory: undefined,
    echo: true,
    env: undefined,
    machineName: undefined,
    stdin: undefined,
}) => __awaiter(void 0, void 0, void 0, function* () {
    let machineconfig = '';
    if (options.machineName) {
        machineconfig = yield new dockermachine_cli_js_1.DockerMachine()
            .command(`config ${options.machineName}`)
            .then((data) => data.machine.config);
    }
    const execCommand = `docker ${machineconfig} ${command}`;
    const execOptions = {
        cwd: options.currentWorkingDirectory,
        env: Object.assign({ DEBUG: '', HOME: process.env.HOME, PATH: process.env.PATH }, options.env),
        maxBuffer: 200 * 1024 * 1024,
    };
    const raw = yield new Promise((resolve, reject) => {
        var _a, _b, _c, _d;
        const childProcess = exec(execCommand, execOptions, (error, stdout, stderr) => {
            if (error) {
                return reject(Object.assign(new Error(`Error: stdout ${stdout}, stderr ${stderr}`), Object.assign(Object.assign({}, error), { stdout, stderr, innerError: error })));
            }
            resolve(stdout);
        });
        if (options.stdin) {
            (_a = childProcess.stdin) === null || _a === void 0 ? void 0 : _a.write(options.stdin);
            (_b = childProcess.stdin) === null || _b === void 0 ? void 0 : _b.end();
        }
        if (options.echo) {
            (_c = childProcess.stdout) === null || _c === void 0 ? void 0 : _c.on('data', (chunk) => {
                process.stdout.write(chunk.toString());
            });
            (_d = childProcess.stderr) === null || _d === void 0 ? void 0 : _d.on('data', (chunk) => {
                process.stderr.write(chunk.toString());
            });
        }
    });
    return extractResult({
        command: execCommand,
        raw,
    });
});
exports.dockerCommand = dockerCommand;
class Docker {
    constructor(options = {
        machineName: undefined,
        currentWorkingDirectory: undefined,
        echo: true,
        env: undefined,
        stdin: undefined,
    }) {
        this.options = options;
    }
    command(command, callback) {
        return (0, nodeify_ts_1.default)((0, exports.dockerCommand)(command, this.options), callback);
    }
}
exports.Docker = Docker;
class Options {
    constructor(machineName, currentWorkingDirectory, echo = true, env, stdin) {
        this.machineName = machineName;
        this.currentWorkingDirectory = currentWorkingDirectory;
        this.echo = echo;
        this.env = env;
        this.stdin = stdin;
    }
}
exports.Options = Options;
//# sourceMappingURL=index.js.map